import json

from django.contrib.postgres.forms import JSONField

from polyquack.translation import Translatable


class TranslatablePgJSONFormField(JSONField):
    def prepare_value(self, value):
        # Return translations when we have a Translatable object
        if isinstance(value, Translatable):
            return json.dumps(value.translations)
        # Dicts default to single-quotes, which is not valid JSON
        if isinstance(value, dict):
            return json.dumps(value)
        return value



"""
Django template filters.
"""

from django import template

register = template.Library()


@register.filter(name="get_translation")
def get_translation(translatable, language):
    """
    Usage:
        {% load translate_filters %}
        {{ translatable_instance | get_translation: "en-ca" }}
    """
    return translatable.translate(language)



"""
Some useful decorators for use with Ariadne GraphQL.
"""

from django.utils import translation


def translate_resolved_to_request_language(resolver):
    """
    This decorator wraps a resolver function that returns a Translatable instance,
    and gets the appropriate translation for the request's language (according to 
    Django's rules).
    """

    def wrapper_translate(*args, **kwargs):
        _, info, *__ = args
        resolved = resolver(*args, **kwargs)
        return resolved.translate(translation.get_language_from_request(info.context))

    return wrapper_translate




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['polyquack_django', 'polyquack_django.templatetags']

package_data = \
{'': ['*']}

install_requires = \
['Django>2.2,<4', 'polyquack>=0.2,<0.3']

setup_kwargs = {
    'name': 'polyquack-django',
    'version': '0.1.2',
    'description': 'Django utilities for polyglots',
    'long_description': '# polyquack-django\n\npolyquack translation and pluralization utilities and fields for Django.',
    'author': 'Patryk Bratkowski',
    'author_email': 'git@patryk.tech',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://gitlab.com/patryk-media/polyquack-django',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6',
}


setup(**setup_kwargs)



"""
This module defines some custom model fields for Django.
"""
# import logging

# Use with logging (if settings.DEBUG: logger.debug(...))
# from django.conf import settings

from django.contrib.postgres.fields import JSONField

from polyquack.translation import Translatable

from .form_fields import TranslatablePgJSONFormField

# logger = logging.getLogger("django")


class TranslatablePgJSONField(JSONField):
    """
    This class offers a Translatable PostgreSQL JSONField.
    """

    description = "This class offers a Translatable PostgreSQL JSONField."

    def __init__(
        self, match_level: str = None, default_language: str = None, *args, **kwargs,
    ):
        self.match_level = match_level
        self.default_language = default_language
        super().__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        if self.match_level is not None:
            kwargs["match_level"] = self.match_level
        if self.default_language is not None:
            kwargs["default_language"] = self.default_language
        return name, path, args, kwargs

    def from_db_value(self, value, expression, connection):
        if value is None:
            return value
        return Translatable(
            translations=value,
            match_level=self.match_level,
            default=self.default_language,
        )

    def formfield(self, **kwargs):
        defaults = {"form_class": TranslatablePgJSONFormField}
        defaults.update(kwargs)
        return super().formfield(**defaults)
